# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""RSS Reader app autopilot tests."""

from __future__ import absolute_import


import unittest

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Is, Not

from autopilot.introspection.dbus import StateNotFoundError
from ubuntu_rssreader_app.tests import RssReaderAppTestCase

from time import sleep

class TestMainWindow(RssReaderAppTestCase):

    def setUp(self):
        super(TestMainWindow, self).setUp()

        self.assertThat(self.main_view.visible, Eventually(Equals(True)))

        #wait for any updates to finish before beginning tests
        self.assertThat(self.main_view.get_network_activity, Eventually(Not(Is(None))))
        self.assertThat(self.main_view.get_network_activity, Eventually(Is(None)))

    def _add_feed_and_topic(self, feed, topicName):
        """test add feed and topic"""
        print "starting test add feed and topic"
        #open toolbar
        print "opening toolbar"
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        print "clicking addreadsbutton"
        toolbar.click_button("addreadsbutton")

        #click on add feeds popover action
        print "clicking + Add feeds"
        addReadsPopover = self.main_view.get_action_popover()
        self.main_view.run_action(addReadsPopover, "+ Add feeds")

        #verify Append Feed  page is open
        print "asserting append page"
        appendpage = lambda: self.main_view.get_append_page().visible
        self.assertThat(appendpage, Eventually(Equals(True)))

        #type feed in input field
        print "inputting feed"
        self._input_new_feed(feed)

        #click on checkbox to confirm feed
        print "asserting checkbox"
        self.assertThat(self.main_view.get_append_page_feed, Eventually(Not(Is(None))))
        print "grabbing checkbox"
        checkboxBtn = self.main_view.get_append_page_feed()
        print "clicking checkbox"
        self.pointing_device.click_object(checkboxBtn)
        print "asserting checkbox checked"
        self.assertThat(checkboxBtn.checked, Eventually(Equals(True)))

        #click on Next button
        print "asserting next"
        self.assertThat(self.main_view.get_next_button, Eventually(Not(Is(None))))
        print "grabbing next"
        nextButton = self.main_view.get_next_button()
        print "clicking next"
        self.pointing_device.click_object(nextButton)

        #verify choose topic page has opened
        print "asserting choose topic page"
        chossetopicpage = lambda: self.main_view.get_topics_page().visible
        self.assertThat(chossetopicpage, Eventually(Equals(True)))

        #get current topics
        print "grabbing topics"
        topics = self.main_view.get_choose_topics_items()
        numTopics = len(topics)
        print "found " + str(numTopics) + " topics"
        for topic in topics:
            print "Topic: " + str(topic.text)

        #add a new topic
        print "input topic"
        self._input_new_topic(topicName)

        #A new topic is now automatically selected as the topic
        #select new topic
        #print "assert new topic"
        #newNumTopics = len(self.main_view.get_choose_topics_items)
        #self.assertThat(newNumTopics, Eventually(Equals(numTopics + 1)))
        #print "grab new topic"
        #topics = self.main_view.get_choose_topics_items()
        #numTopics = len(topics)
        #print "found " + str(numTopics) + " topics"
        #for topic in topics:
        #    print "Topic: " + str(topic.text)
        #newTopic = self.main_view.get_choose_topics_items()[-1]
        #newTopic = topics[-1]
        #print "click new topic, " + str(newTopic.text)
        #self.pointing_device.click_object(newTopic)
        #print "topic clicked"

        #wait for any updates to finish before beginning tests
        #this transition currently locks the device https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1221893
        #print "check for network updates"
        #self.assertThat(self.main_view.get_network_activity, Eventually(Not(Is(None))))
        #print "assert no network updates"
        #self.assertThat(self.main_view.get_network_activity, Eventually(Is(None)))
        #print "assert we're back on main view"
        #self.assertThat(self.main_view.visible, Eventually(Equals(True)))
        #print "finished with add_feed_topic"

    def _remove_feed(self, feedName, topicName):
        print "removing feed"
        #open toolbar
        print "opening toolbar"
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        print "clicking edit topics"
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        print "assert feedlist page"
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #select topic
        print "grabbing topic"
        self._get_topic(topicName)

        #select feed
        print "assert feed"
        self.assertThat(lambda: self.main_view.get_feedlist_feed(feedName), Eventually(Not(Is(None))))
        print "grabbing feed"
        feed = self.main_view.get_feedlist_feed(feedName)
        print "clicking feed"
        self.pointing_device.click_object(feed)

        #verify we are on the Edit Feed page
        print "assert editfeedpage"
        editFeedPage = self.main_view.get_edit_feed_page()
        self.assertThat(editFeedPage.visible, Eventually(Equals(True)))

        #press delete button
        print "assert delete button"
        self.assertThat(self.main_view.get_delete_button, Eventually(Not(Is(None))))
        print "grab delete button"
        deleteButton = self.main_view.get_delete_button()
        print "click delete button"
        self.pointing_device.click_object(deleteButton)

        #verify we are on the Management page
        print "assert feedlistpage"
        feedlistpage = lambda: self.main_view.get_feed_management_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #check topic
        print "assert topic remains"
        self.assertThat(lambda: self.main_view.get_feedlist_topic(topicName), Eventually(Not(Is(None))))

        #check feed has been removed
        print "assert removed feed"
        self.assertThat(lambda: self.main_view.get_feedlist_feed(feedName), Eventually(Equals(None)))

    def _remove_topic(self, topicName):
        #this been quasi converted to the sdk, but does not work as-is
        #see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1211631

        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar Button
        toolbar.click_button("edittopicsbutton")

        #verify we are on the Management page
        feedlistpage = self.main_view.get_feed_management_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select topic
        topic = self._get_topic(topicName)

        topicsPage = self.main_view.get_manage_topics_page()
        self.assertThat(topicsPage, Eventually(Not(Is(None))))

        addATopicItem = lambda: self.main_view.get_add_a_topic()
        self.assertThat(addATopic, Eventually(Not(Is(None))))

        #swipe to delete topic
        #we have to consider the top tab offset, then create a swipe operation across the width of the page
        startX = int(topicsPage.x + topicsPage.width * 0.20)
        stopX = int(topicsPage.x + topicsPage.width * 0.80)
        tabHeightOffset = qmlView.height - topicsPage.height + addATopicItem.height
        lineY = int(qmlView.y + topic.y +  tabHeightOffset)

        #swipe to remove the topic
        self.pointing_device.move(startX, lineY)
        self.pointing_device.drag(startX, lineY, stopX, lineY)

        #check that it is gone
        topic = lambda: self.main_view.get_feedlist_topic(topicName)
        self.assertThat(topic.visible, Eventually(Equals(False)))

    def _input_new_feed(self, rssFeed):
        inputField = self.main_view.get_append_feed_input_box()
        return self._safe_enter_text(inputField, rssFeed)

    def _input_new_topic(self, topic):
        inputField = self.main_view.get_new_topic_input_box()
        return self._safe_enter_text(inputField, topic)

    def _safe_enter_text(self, inputField, text, timeout = 10):
        #poll clicking for focus for lack of better method, due to screen switching animation
        #is there a property we can use instead?
        poll = 0
        while inputField.focus != True and poll < timeout:
            self.pointing_device.click_object(inputField)
            sleep(1)
            poll += 1
        self.keyboard.type(text)
        self.assertThat(inputField.text, Eventually(Equals(text)))
        self.keyboard.press_and_release("Enter")
        return inputField

    def _get_topic(self, topicName, timeout = 10):
        print "_get_topic: assert topics"
        self.assertThat(lambda: self.main_view.get_feedlist_topic(topicName), Eventually(Not(Is(None))))
        print "_get_topic: grab topic"
        topic = self.main_view.get_feedlist_topic(topicName)
        print "_get_topic: click topic"

        poll = 0
        while topic.isExpended == False and poll < timeout:
            print "_get_topic: loop click topic"
            self.pointing_device.click_object(topic)
            sleep(1)
            poll += 1
        return topic

    def test_add_remove_feed_and_topic(self):
        """ test add and remove feed and topic"""

        #add topic and feed
        #TODO, try several different feeds
        feed = "http://www.canonical.com/rss.xml"
        feedName = "Canonical"
        topicName = "CanonicalTopic"

        self._add_feed_and_topic(feed, topicName)

        #remove added feed
        print "starting remove feed"
        #12 sept 2013,
        #this transition currently locks the device https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1221893
        #self._remove_feed(feedName, topicName)

        #remove added topic
        #12 Aug 2013, removing topic broken for now
        #test commented until fixed, see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1211631
        #self._remove_topic()


    @unittest.skip("Topic page broken, see https://bugs.launchpad.net/ubuntu-rssreader-app/+bug/1206248")
    def test_view_feeds(self):
        """test view feeds"""

        #-----> view all news:
        #open toolbar
        toolbar = self.main_view.open_toolbar()

        #click on add reads toolbar button
        toolbar.click_button("List view")

        #verify we are on the Feeds page
        feedlistpage = self.main_view.get_feedlist_page()
        self.assertThat(feedlistpage.visible, Eventually(Equals(True)))

        #select All articles
        allFeeds = self.main_view.get_all_feeds()
        self.assertThat(allFeeds, NotEquals("None"))
        self.pointing_device.click_object(allFeeds )

        #verify we are on the All articles page
        articlelistpage = self.main_view.get_articlelist_page()
        self.assertThat(articlelistpage.visible, Eventually(Equals(True)))

        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #-----> view individual feed:
        #add topic and feed
        self._add_feed_and_topic()

        #select Canonical feed
        canonicalFeed = lambda: self.main_view.get_canonical_feed_labelvisual()
        self.assertThat(canonicalFeed, Eventually(Not(Is(None))))
        self.pointing_device.click_object(canonicalFeed)

        #verify we are on the Canonical page
        header = lambda: self.main_view.get_header()[0].title
        self.assertThat(header, Eventually(Equals("Canonical")))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on back toolbar button
        toolbar.click_button("Back")

        #verify we are on the Feeds page
        feedlistpage = lambda: self.main_view.get_feedlist_page().visible
        self.assertThat(feedlistpage, Eventually(Equals(True)))

        #toggle toolbar
        toolbar = self.main_view.open_toolbar()

        #click on shorts toolbar button
        toolbar.click_button("shorts")

        #verify we are on the Shorts page
        shortsPage = lambda: self.main_view.switch_to_tab_by_index(0).visible
        self.assertThat(shortsPage, Eventually(Equals(True)))

        #remove added Canonical topic
        self._remove_topic()

        #remove added Canonical feed
        self._remove_feed()
